%  Function to return position and velocity in Earth-Centered J2000 Frame using the
% atbSetEpoch call from the AeroToolBox.  Input is Julian Date instead of the year, day
% date etc.. format of atbSetEpoch routine.
% 
% Mike Loucks - Space Exploration Engineering, Inc.
%

function [pos, vel] = jdephem(pos, vel, jdate);

% break down jdate into integer julian date and fractional day

jdix = double(int32(jdate));
jdfx = jdate - jdix;

jdfx = jdfx + 0.5;
if jdfx >= 1
    jdfx = jdfx - 1;
    jdix = jdix + 1;
end

if jdix > 2299160
    a =(4.0*(jdix-1867216.0)-1)/146097.0;
    b = jdix + 1 + a + a/4;
else
    b = jdix;
end

c = b + 1524;
d = (20*c-2442)/7305;
e = 365*d + d/4;
g = (10*(c-e)-1)/306;

day = c - e - 30*g - (6*g)/10;

if g <= 13 
    month = int32(g-1);
else
    month = int32(g-13);
end

if month <= 2
    year = int32(d - 4715);
else
    year = int32(d - 4716);
end

% computer the hour, minute, second

x = jdfx*24;
hour = double(int32(x));
x = (x - hour)*60.0;
min = double(int32(x));
sec = (x - min)*60.0;
    
atbSetEpoch(year,month,day,hour,min,sec,'z',0);
[pos, vel] = atbCbEphemeris('Earth', 0);

return