%SUBROUTINE LAMGTL (TLAMB)
% Dec 21, 1990
% Subroutine Lamgtl obtained from Alan Klumpp of JPL
% Log of Changes, Version of 90 12 21
%
%    1.   90 10 26, Allan Klumpp:  Renamed TLAMB LAMGTL.
%
%    2.   90 12 12, Allan Klumpp:  Added IGTL to calling sequences.
%         IGTL counts the number of times LAMGTL is called.
%

function [t, dt, d2t, d3t, igtl] = lamgtl(m, q, qsqfm1, x, n, igtl)

sw = 0.4;

igtl = igtl + 1;

dt = 0.0;
dt2 = 0.0;
dt3 = 0.0;

lm1 = n == -1;
l1 = n >= 1;
l2 = n >= 2;
l3 = n == 3;

qsq = q*q;
xsq = x*x;
u = (1.0 - x)*(1.0 + x);

if  ~ lm1
    % (Needed if series, and otherwise useful when z = 0)
    dt = 0.0;
    d2t = 0.0;
    d3t = 0.0;
end
      
if lm1 | m > 0 | x < 0.0 | abs(u) > sw
    % Direct computation (not series)
    y = sqrt(abs(u));
    z = sqrt(qsqfm1 + qsq*xsq);
    qx = q*x;
    
    if qx <= 0.0
        a = z - qx;
        b = q*z - x;
    end
    
    if qx < 0.0 & lm1
        aa = qsqfm1/a;
        bb = qsqfm1*(qsq*u - xsq)/b;
    end
    
    if qx == 0.0 & lm1 | qx > 0
        aa = z + qx;
        bb = q*z + x;
    end
    
    if qx > 0.0
        a = qsqfm1/aa;
        b = qsqfm1*(qsq*u - xsq)/bb;
    end

    if ~ lm1
        if qx*u >= 0.0
            g = x*z + q*u;
        else
            g = (xsq - qsq*u)/(x*z - q*u);
        end
        f = a*y;

        if x <= 1.0
            t = m*pi + atan2(f, g);
        else
            if f > sw
                t = log(f + g);
            else
                fg1 = f/(g + 1.0);
                term = 2.0*fg1;
                fg1sq = fg1*fg1;
                t = term;
                twoi1 = 1.0;
                twoi1 = twoi1 + 2.0;
                term = term*fg1sq;
                told = t;
                t = t + term/twoi1;
                
                while t ~= told
                    twoi1 = twoi1 + 2.0;
                    term = term*fg1sq;
                    told = t;
                    t = t + term/twoi1;
                end %while
			  
% (Continue looping for inverse tanh)

            end %if
        end %else if
          
        t = 2.0*(t/y + b)/u;
          
        if l1 & z ~= 0.0
            qz = q/z;
            qz2 = qz*qz;
            qz = qz*qz2;
            dt = (3.0*x*t - 4.0*(a + qx*qsqfm1)/z)/u;
            
            if l2 
                d2t = (3.0*t + 5.0*x*dt + 4.0*qz*qsqfm1)/u;
            end
            
            if l3
                d3t = (8.0*dt + 7.0*x*d2t - 12.0*qz*qz2*x*qsqfm1)/u;
            end
            
        end
            
    else
      t = 0;
      dt = b;
      d2t = bb;
      d3t = aa;
    end % if
        
else
	   
% Compute by series
  u0i = 1.0;
  if l1
    u1i = 1.0;
  end
        
  if l2
    u2i = 1.0;
  end
        
  if l3
    u3i = 1.0;
  end
        
  term = 4.0;
  tq = q*qsqfm1;
  i = 0;
        
  if (q < 0.5) 
    tqsum = 1.0 - q*qsq;
  end
        
  if (q >= 0.5) 
    tqsum = (1.0/(1.0 + q) + q)*qsqfm1;
  end
        
  ttmold = term/3.0;
  t = ttmold*tqsum;
		
% (Start of loop)
  told = t-1.0;   % force t ~= told to get one pass through
       
  while i < n | t ~= told
    i = i + 1;
    p = i;
    u0i = u0i*u;
        
    if l1 & i > 1
      u1i = u1i*u;
    end
            
    if l2 & i > 2
      u2i = u2i*u;
    end
            
    if l3 & i > 3 
      u3i = u3i*u;
    end
       
    term = term*(p - 0.5)/p;
    tq = tq*qsq;
    tqsum = tqsum + tq;
    told = t;
    tterm = term/(2.0*p + 3.0);
    tqterm = tterm*tqsum;
    t = t - u0i*((1.5*p + 0.25)*tqterm/(p*p - 0.25)- ttmold*tq);
    ttmold = tterm;
    tqterm = tqterm*p;
        
    if l1
      dt = dt + tqterm*u1i;
    end
            
    if l2 
      d2t = d2t + tqterm*u2i*(p - 1.0);
    end
            
    if l3 
      d3t = d3t + tqterm*u3i*(p - 1.0)*(p - 2.0);
    end
 
  end % end of while loop
% (End of loop)

  if l3 
    d3t = 8.0*x*(1.5*d2t - xsq*d3t);
  end
            
  if l2
    d2t = 2.0*(2.0*xsq*d2t - dt);
  end
            
  if (l1) 
    dt = -2.0*x*dt;
  end
  
  t = t/xsq;
  
end % if lml | m>0 end

return
