% SUBROUTINE LAMGXL (XLAMB)
% Dec 24, 1990
% Subroutine Lamgtl obtained from Alan Klumpp of JPL
% Log of Changes, Version of 90 12 24
%
%    1.   90 10 26, Allan Klumpp:  Renamed XLAMB LAMGXL.
%
%    2.   90 12 12, Allan Klumpp:  Added IGTL to calling sequences.
%         IGTL counts the number of times LAMGTL is called.
%
% begin subroutine lamgxl
  
function [x,xpl,n,igtl] = lamgxl(m, q, qsqfm1, tin, n, igtl)

tol = 3.0e-7;
c0  = 1.7;

c1=0.5;
c2=0.03;
c3=0.15;
c41=1.0;
c42=0.24;

x = 0.0;
xpl = 0.0;
t0 = 0.0;
t = 0.0;
thr2 = atan2(qsqfm1, 2.0*q)/pi;

three = 0;

if m == 0 & ~three
    
   %  Single-rev starter from  t (at x = 0) & bilinear (usually)
    n = 1;
    
    [t0, dt, d2t, d3t, igtl] = lamgtl(m, q, qsqfm1, 0.0, 0, igtl);
   
    tdiff = tin - t0;
   
    if tdiff < 0.0
        x = t0*tdiff/(-4.0*tin);
		  
% (-4 is the value of dt, for x = 0)

    else
    
        x = -tdiff/(tdiff + 4.0);
        w = x + c0*sqrt(2.0*(1.0 - thr2));
    
        if (w < 0.0)
            x = x - sqrt(d8rt(-w))*(x + sqrt(tdiff/(tdiff + 1.5*t0)));
        end
    
        w = 4.0/(4.0 + tdiff);
        x = x*(1.0 + x*(c1*w - c2*x*sqrt(w)));
    end
        
else
	   
% With multirevs, first get t(min) as basis for starter

    xm = 1.0/(1.5*(m + 5.0e-1)*pi);
    if (thr2 < 0.5) 
        xm = d8rt(2.0*thr2)*xm;
    end
    
    if (thr2 > 0.5) 
        xm = (2.0 - d8rt(2.0 - 2.0*thr2))*xm;
    end
		
% (Starter for tmin)

    for i=1:12
        
        [tmin, dt, d2t, d3t, igtl] = lamgtl(m, q, qsqfm1, xm, 3, igtl);
        
        if  d2t == 0.0
            break;
        end 
        
        xmold = xm;
        xm = xm - dt*d2t/(d2t*d2t - dt*d3t/2d0);
%
        xtest = abs(xmold/xm - 1d0);
        
        if xtest <= tol
            break;
        end
        
    end  %for 
    
    if xtest > tol | d2t == 0.0
        n = -1;
        return
    end
		
% (Break off & exit if tmin not located - should never happen)
%  now proceed from t(min) to full starter

    tdiffm = tin - tmin;
    
    if tdiffm < 0.0
        n = 0;
        return
%          (exit if no solution with this m)
    elseif tdiffm == 0.0
        x = xm;
        n = 1;
        return;
%          (exit if unique solution already from x(tmin))
    else
        n = 3;
        if d2t == 0.0
            d2t = 6d0*m*pi;
        end
        
        x = sqrt(tdiffm/(d2t/2.0 + tdiffm/(1.0 - xm)^2));
        w = xm + x;
        w = w*4.0/(4.0 + tdiffm) + (1.0 - w)^2;
        x = x*(1.0 - (1.0 + m + c41*(thr2 - 0.5))/(1.0 + c3*m)*x*(c1*w + c2*x*sqrt(w))) + xm;
        d2t2 = d2t/2.0;

        if (x >= 1.0) 
            n = 1;
            three = 1;
        end
 
% (No finite solution with x > xm)

    end

end
	  
% (Now have a starter, so proceed by halley)

k = 1;

while k==1               % Infinite loop that must be exited ultimately by one of the return statements.

    if ~three
        for i=1:3
            
            [t, dt, d2t, d3t, igtl] = lamgtl(m, q, qsqfm1, x, 2, igtl);
            t = tin - t;
    
            if dt ~= 0.0
                x = x + t*dt/(dt*dt + t*d2t/2.0);
            end
    
        end %for

        if n ~= 3
            return
        end
	  
% (Exit if only one solution, normally when m = 0)

        n = 2;
        xpl = x;
    end  % three if block
	  
% (Second multi-rev starter)
    
    [t0, dt, d2t, d3t, igtl] = lamgtl(m, q, qsqfm1, 0d0, 0, igtl);
    
    tdiff0 = t0 - tmin;
    tdiff = tin - t0;

    if tdiff <= 0
        x = xm - sqrt(tdiffm/(d2t2 - tdiffm*(d2t2/tdiff0- 1d0/xm^2)));
    else
        x = -tdiff/(tdiff + 4.0);
        ij = 200;
        w = x + c0*sqrt(2.0*(1.0 - thr2));
    
        if w < 0.0
            x = x - sqrt(d8rt(-w))*(x + sqrt(tdiff/(tdiff + 1.5*t0)));
            w = 4.0/(4.0 + tdiff);
            x = x*(1.0 + (1.0 + m + c42*(thr2 - 0.5))/(1d0 + c3*m)*x*(c1*w - c2*x*sqrt(w)));
        
            if (x <= -1.0)
                n = n - 1;     % (No finite solution with x < xm)

                if (n ~= 1) 
                x = xpl;
                end
          
            end 
        end
    end

    three = 0;

end  % while loop end