function testcom

%This function demonstrates several calls to the Astrogator Object model
%available though matlab and Microsoft COM automation.  It includes to
%utility functions at the end to make it easier to access targeter profiles
%and results

%Created 1/29/06 by Mike Loucks, Space Exploration Engineering Corp.
%www.see.com, 360-378-7168

    % Specify file name for reading in a scenario file later.
	% file = 'C:\Documents and Settings\Mike Loucks\My Documents\STK 7\Single_Segment_Test\GatorTest.sc'; 
    
    file = 'C:\Documents and Settings\Mike Loucks\My Documents\STK 7\Earth To Moon Tool Scenarios\Direct Surveyor-Type Kinetic\Lunar_Lander.sc';
    
    % Open STK Application.  Right now I don't know how to open an existing
    % instance, so we have to open our own instance and load our scenario.
    
    g_uiApp = actxserver('STK7.application');
    
    % get the 'personality2' in order to open files.  This sort of stuff
    % (File I/O, etc) needs to be done in the new STK object model, which
    % requires 'personality2'.  Once open, we make the application visible
    % and load the file.
    g_stk = g_uiApp.get('Personality2');
    
    %make the application visible
    g_uiApp.Visible = 1;
    
    %make the application controlable by the user, so it won't disappear
    %after this script runs.
    g_uiApp.UserControl = 1;
    g_stk.LoadScenario(file);
    
    % now we have to access regular 'personality' to use the old object
    % model that astrogator still uses so we can access astrogator properly
    stkOldInterface=g_uiApp.get('Personality');
    
    % open the scenario, and the satellite.  Only one scenario can exist,
    % but multiple satellites can exist.  Don't know if you can refer to
    % satellites by name or not, but numbers work and numbering starts at
    % zero.
    scenario = stkOldInterface.invoke('Scenario', 0);
    satellite = scenario.invoke('Satellite', 0);

    % access the astrogator MCS
    gator=satellite.invoke('GetAstrogatorDefn');
    
    % access the individual segments.  there is a getsegmenbyname function
    % for this, which allows you to use the segment names.
    mtheTargeter = gator.invoke('GetSegmentByName','Target_Sequence');
    
    % Run a segment.  If you want to run the entire MCS, right now there is
    % no com call for that.  You either have to use an old-style connect
    % command, or encase the MCS in either a targeter or a sequence segment
    % and then execute that.  In this case, the MCS is contained within a
    % targeter, so you can run the whole thing that way.  Note that this
    % usage also allows individual segments to be propagated.
    
    % Begin run and end run are functions used when propagating single
    % segments.  Begin run tells Astrogator that the ephemeris is being
    % created, endrun tells it that the ephemeris has been created and
    % closes it.
	tf = gator.invoke('BeginRun');
	state = gator.invoke('RunSegment',mtheTargeter);
	tf = gator.invoke('EndRun');
    
    % find a profile and change the desired result.  First set the name,
    % result and segment name we want
    desired_profile_name = 'RA_Dec'; %Note that names with spaces need to have underscores instead
    desired_result_name = 'Delta_Declination';
    result_segment_name = 'Transfer.Prop-to-Lunar-SOI';
    
    %Call a matlab function getprofilebyname that we created for this
    %example to extract a pointer to the desired profile
    desired_profile = getprofilebyname(mtheTargeter, desired_profile_name);
    
    %Call a matlab function getresultbyname that we created for this
    %example to extract a pointer to the desired result
    declintion = getresultbyname(desired_profile, desired_result_name, result_segment_name);
    declintion.set('Desired',0.5235987760);
    
    % get an element from the final state properties of a segment.  Most
    % segments have several sets of default elements, such as those that
    % show up in the summary.  Note that the coordinate system here for the
    % SMA is that set by the user in the properties page for the segment.
    msegment = gator.invoke('GetSegmentByName','Approach-Moon');
    k = msegment.invoke('FinalState.Keplerian.sma');
    j = k.invoke('getin','km');
    SMA = j.invoke('Value');
   
    % get any calc object from a segment, even if it hasn't been
    % added as a result.  The 'getresultvalue' method can be invoked on any
    % segment, using any calc object.
    k = msegment.invoke('GetResultValue', 'Altitude');
    j = k.invoke('getin','km');
    Altitude = j.invoke('Value');
    
    %Turn off a profile by changing its mode
    desired_profile_name = 'RMag_Earth_on';
    desired_profile = getprofilebyname(mtheTargeter, desired_profile_name);
    %Note that mode setting requires spaces to match, not underscores
    desired_profile.set('Mode','Not Active');
    
    end

function[save] = getprofilebyname(targeter, profilename)

% function to retrieve a pointer to a profile based on the profile name

    mm = 0;
	while mm < 100     
        try 
            profile_1 = targeter.invoke('profiles',mm);
            k = profile_1.invoke('ComponentName');
            current_profilename = k.invoke('Value');
            if strcmp(current_profilename,profilename)
                save = profile_1;
                mm = 100;
                %disp 'Found Profile Name', profilename, 'Index ', save;
            end
            mm = mm+1;
        catch
            mm = 100;
            disp 'Error finding Profile Name', profilename;
        end    
    end     
    
end


function[save] = getresultbyname(profile, result_name, ParentSegmentName)

% function to retrieve a pointer to a result based on the result name,
% profile name and segment name
    mm = 0;
	while mm < 100     
        try 
            result = profile.invoke('Results',mm);
            k = result.invoke('ComponentName');
            current_resultname = k.invoke('Value');
            
            k = result.invoke('ParentSegmentName');
            current_ParentSegmentName = k.invoke('Value');
            
            if strcmp(current_resultname,result_name) & strcmp(current_ParentSegmentName,ParentSegmentName)
                save = result;
                mm = 100;
                %disp 'Found Result Name', result_name, 'Index ', save;
            end
            mm = mm+1;
        catch
            mm = 100;
            disp 'Error finding Result Name', result_name;
        end    
    end     
    
end

function[save] = getcontrolbyname(profile, control_name, ParentSegmentName)

% function to retrieve a pointer to a result based on the result name,
% profile name and segment name
    mm = 0;
	while mm < 100     
        try 
            control = profile.invoke('Controls',mm);
            k = control.invoke('ComponentName');
            current_controlname = k.invoke('Value');
            
            k = control.invoke('ParentSegmentName');
            current_ParentSegmentName = k.invoke('Value');
            
            if strcmp(current_controlname,result_name) & strcmp(current_ParentSegmentName,ParentSegmentName)
                save = result;
                mm = 100;
                %disp 'Found Result Name', result_name, 'Index ', save;
            end
            mm = mm+1;
        catch
            mm = 100;
            disp 'Error finding Control Name', control_name;
        end    
    end     
    
end